/**
 * Base class for defining characters and their elements.
 */
export declare abstract class Characters extends Object {
    /**
     * Round values to specified decimal places.
     */
    precision: number;
    /**
     * The stroke size of the lines.
     */
    abstract stroke: number;
    /**
     * Character and their elements encode bits.
     */
    protected readonly _characters: {
        [c: string]: number | null;
    };
    /**
     * Character elements.
     */
    protected abstract readonly _elements: (readonly number[])[];
    /**
     * Margin values.
     */
    margin: {
        left: number;
        right: number;
        top: number;
        bottom: number;
    };
    /**
     * Characters constructor.
     */
    constructor();
    /**
     * Character width, with stroke.
     *
     * @returns Width size.
     */
    abstract get characterWidth(): number;
    /**
     * Character height, with stroke.
     *
     * @returns Height size.
     */
    abstract get characterHeight(): number;
    /**
     * Character center X position, with margins.
     *
     * @returns X position.
     */
    get centerX(): number;
    /**
     * Character center Y position, with margins.
     *
     * @returns Y position.
     */
    get centerY(): number;
    /**
     * Total width, with margin, rounded up to the next pixel.
     *
     * @returns Total width.
     */
    get width(): number;
    /**
     * Total height, with margin, rounded up to the next pixel.
     *
     * @returns Total height.
     */
    get height(): number;
    /**
     * Round value to the configured precision.
     *
     * @param n Value to be rounded.
     * @returns Rounded value.
     */
    round(n: number): number;
    /**
     * Get character encode bits.
     *
     * @param character The character to encode.
     * @returns Encode bits or null.
     */
    getCharacter(character: string): number | null;
    /**
     * Get character elements for encode bits.
     * Each length will be 3 (circle), 4 (line), or >4 (path).
     * Circle: `[r, cx, cy]`
     * Line: `[x1, y1, x2, y2]`
     * Path: `[x1, y1, x2, y2, ...]` (NaN ending closes path)
     * Values are padded and made absolute.
     *
     * @param bits Encode bits.
     * @returns Element coordinates or null.
     */
    getElements(bits: number): number[][];
    /**
     * Get character elements for a character.
     *
     * @param character The character to encode.
     * @returns Element coordinates or null.
     */
    getCharacterElements(character: string): number[][] | null;
}
